;################################################################### LIBHC.INC
; Purpose.....:	LIBH: COMBO-chip functions
; History.....: 11.01.2024: Created
;#############################################################################

;===================================================================== hCTAS()
; Purpose.....:	Setup combochip timer A
; Input.......: A= timer interval, one of hcCTxx
;		CY=1: initialize, CY=0: de-initialize and stop
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hCTASx: h)ardware C)ombo Timer A) S)etup
;=============================================================================
hCTAS::		PUSH	HL
		PUSH	DE
		PUSH	BC

		JR	NC,hCTASD	; CY=0, de-init

;-------------------------------------------------------- clear callback table
		PUSH	AF
		LD	B,hvCTAE-hvCTAA	; B= length of table
		LD	HL,hvCTAA	; HL= start of table
		XOR	A		; A= fill value
hCTASC:		LD	(HL),A
		INC	HL
		DJNZ	hCTASC
		POP	AF

;----------------------------------------------- copy interrupt code to hi-ram
		LD	HL,hcCIY	; HL= int code address in lo-ram
		LD	DE,hcICIS	; DE= int code address in hi-ram
		LD	BC,hcICIE-hcICIS; BC= code length
		LDIR

;----------------------------------------------------- setup interrupt vectors
hcCTAV		EQU	hcIVT+hcICV	; table start Combo interrupts
hcCTAA		EQU	hcCTAV+hcICTA	; table offset timerA interrupt

		LD	HL,hICTA	; HL= timerA int handler address
		LD	(hcCTAA),HL	; copy to vector table

;--------------------------------------------------------------- setup timer A
		LD	HL,hcCTAT	; HL= table address
		LD	D,0
		SLA	A		; table has two elements per item
		LD	E,A
		ADD	HL,DE		; HL= address of item to use

		LD	E,00010000B	; timer A interrupt enable, stop
		LD	A,(HL)		; A= prescaler value
		OR	E		; A= control register contents
		OUT	(hcCPAC),A	; write to timer A control register
		LD	(hvCTAC),A	; save value for start/stop timer
 
		INC	HL
		LD	A,(HL)		; A= time constant
		OUT	(hcCPAD),A	; write to time constant register

;------------------------------------------------------------ setup interrupts
		LD	A,00010000B	; interrupt mask, timer A
		OUT	(hcCPIM),A	; write to combo int mask register

hcCTAI		EQU	LOW hcIVT+LOW hcICV
		LD	A,hcCTAI	; lowbyte of int vector table
		OR	00000000B	; vector mode, disable INT1 and INT2	
		OUT	(hcCPIC),A	; write to combo int control reg
		JR	hCTASX		; exit

;------------------------------------------------ de-initialize and stop timer
hCTASD:		LD	A,00000000B	; disable timer A interrupts and stop
		OUT	(hcCPIC),A

		LD	A,00000000B	; reset interrupt mask
		OUT	(hcCPIM),A

hCTASX:		POP	BC
		POP	DE
		POP	HL
		RET

;------------------------------------------------------------------- variables
hvCTAC:		DB	0		; image of control register

;------------------------------------------- prescaler and time constant table
hcCTAT:		DB	10100000B, 59	; 1mS,  p=40  t=59
		DB	11000000B, 42	; 2mS,  p=100 t=42
		DB	11000000B,105	; 5mS,  p=100 t=105
		DB	11100000B,105	; 10mS, p=200 t=105
		DB	11100000B,210	; 20mS, p=200 t=210
;================================================================= eoF hCTAS()

;===================================================================== hCTAR()
; Purpose.....:	Start/stop combo timer A
; Input.......: CY=1: start, CY=0: stop
; Registers...: All saved
; Requires....: Nothing
; Labels......: hCTARx: h)ardware C)ombo Timer A) R)un
;=============================================================================
hCTAR::		PUSH	AF

		LD	A,(hvCTAC)	; A= control register image
		SET	0,A		; preset to start
		JR	NC,hCTARO	; CY=0, stop
		JR	hCTARW

hcTARO:		RES	0,A		; stop timer

hcTARW:		LD	(hvCTAC),A	; save image
		OUT	(hcCPAC),A	; write to timer A control reg

		POP	AF
		RET
;================================================================= eoF hCTAR()

;===================================================================== hCTAC()
; Purpose.....:	Register/deregister timer callback function
; Input.......: HL= Address of function to register/deregister
;		DE= number of calls to do, 0= delete callback, FFFF=infinite
;		BC= timer interval * interrupt rate
; Registers...: IX saved
; Requires....: Nothing
; Labels......: hCTACx: h)ardware C)ombo Timer A) C)allback setup
;=============================================================================
hCTAC::		PUSH	IX
		LD	A,L
		OR	H
		JR	Z,hCTACX	; function address may not be zero

;---------------------- search address in table, if not found, an empty record
		CALL	hCTACS		; search for address in table
		JR	Z,hCTACD	; found, check for delete request
		PUSH	HL
		LD	HL,0
		CALL	hCTACS		; search empty record
		POP	HL
		JR	NZ,hCTACE	; not found, table is full

;-------------------------------------------- check for delete request (DE=0)
hCTACD:		LD	A,E		; check if call counter =0
		OR	D
		JR	NZ,hcTACW	; not zero, write item
		LD	HL,0		; 0 means deleted/unused

hCTACW:		LD	(IX+0),L	; store lowbyte of function address
		LD	(IX+1),H	; store hibyte of function address	
		LD	(IX+2*hcCTAC),E	; store lowbyte of calls counter
		LD	(IX+2*hcCTAC+1),D ; store hibyte of calls counter
		LD	(IX+4*hcCTAC),C	; store lowbyte of interval
		LD	(IX+4*hcCTAC+1),B ; store hibyte of interval
		XOR	A		; success, clear carry
		LD	(IX+6*hcCTAC),A	; clear ints counter
		LD	(IX+6*hcCTAC+1),A
		JR	hCTACX		; and exit
	
hCTACE:		SCF			; table is full, signal with CY
		
hCTACX:		POP	IX
		RET

;--------------------------------------- local sub: search record in data table
; Input.......:	HL= address to search
; Output......:	IX points to table address if found
;		Z=1 if found, 0 if not
;------------------------------------------------------------------------------
hCTACS:		PUSH	BC
		LD	IX,hvCTAA	; IX= addresses table
		LD	B,hcCTAC	; items counter
hCTACL:		LD	A,L		; A= lowbyte of address to search
		CP	(IX+0)		; same as in table?
		JR	NZ,hCTACN	; no, go to next item
		LD	A,H		; A= hibyte of address to search
		CP	(IX+1)		; also same as table?
		JR	Z,hCTACF	; yes, found
hCTACN:		INC	IX		; IX= address of lowbyte next item
		INC	IX
		DJNZ	hCTACL		; go search next
		XOR	A		; item not found
		INC	A		; make sure Z=0
hCTACF:		POP	BC
		RET
;------------------------------------------------------------------- variables
hvCTAA::	DS	2*hcCTAC	; function addresses
hvCTAL:		DS	2*hcCTAC	; calls counters
hvCTAV:		DS	2*hcCTAC	; timer intervals
hvCTAI:		DS	2*hcCTAC	; items interrupt counter
hvCTAE		EQU	$		; end of callback tables
;===================================================================== hCTAC()

;===================================================================== hCTAB()
; Purpose.....:	Execute defined timer callbacks. Must be called from
;		applications main loop
; Registers...: All saved
; Requires....: Nothing
; Labels......: hCTABx: h)ardware C)ombo Timer A) callB)ack
;=============================================================================
hCTAB::		PUSH	AF

		LD	A,(hvICTF)	; get int occurred flag
		BIT	hbCTEC,A	; has it fired?
		JR	Z,hCTABQ	; no, exit
		RES	hbCTEC,A	; yes, reset flag
		LD	(hvICTF),A	; and store it

		PUSH	IX
		PUSH	HL
		PUSH	DE
		PUSH	BC
		
		LD	IX,hvCTAA	; IX= data table base
		LD	B,hcCTAC	; B= loop counter
			
;-------------------------------------------------------- functions calls loop
hCTABL:		LD	A,(IX+0)	; A= lowbyte of function address
		LD	H,(IX+1)	; H= hibyte of function address
		LD	C,A		; save lowbyte
		OR	H		; zero?
		LD	A,C		; recall lowbyte
		CALL	NZ,hCTABC	; no, check and call function
		INC	IX
		INC	IX
		DJNZ	hCTABL		; try next		
		JR	hCTABX		; all done, exit

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~ local sub: a function is to call, check interval
; Input.......:	IX points to function address to call
;		H  hibyte of function address
;		A  lobyte of function address
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
hCTABC:		PUSH	BC
		LD	L,A		; HL= function address
		LD	E,(IX+4*hcCTAC)	; DE= interval
		LD	D,(IX+4*hcCTAC+1)
		LD	C,(IX+6*hcCTAC)	; BC= int counter
		LD	B,(IX+6*hcCTAC+1)
		INC	BC		; bump int counter
		LD	(IX+6*hcCTAC),C	; and save it
		LD	(IX+6*hcCTAC+1),B

		LD	A,E
		CP	C
		JR	NZ,hCTABY	; not equal, no call
		LD	A,D
		CP	B
		JR	NZ,hCTABY	; not equal, no call

;--------------------------------- interval is satisfied, check calls counter
		LD	E,(IX+2*hcCTAC)	; DE= calls counter
		LD	D,(IX+2*hcCTAC+1)
		LD	A,E
		OR	D
		JR	NZ,hCTABI	; counter not zero, check infinite
		LD	DE,0
		CALL	hCTAC		; counter is zero, delete task
		JR	hCTABY		; and quit local sub		

hCTABI:		INC	DE		; DE=0 if infinite calls
		LD	A,E
		OR	D
		DEC	DE		; restore original value
		JR	Z,hCTABR	; is infinite, let counter alone
		DEC	DE		; not infinite, dec counter
		LD	(IX+2*hcCTAC),E	; and store it
		LD	(IX+2*hcCTAC+1),D

;--------------------------------- counter also sastisfied, run the function
hCTABR:		PUSH	IX
		LD	BC,hCTABT	; push return address
		PUSH	BC		; to stack
		JP	(HL)		; run the function

hCTABT:		POP	IX
		XOR	A
		LD	(IX+6*hcCTAC),A	; clear ints counter for task
		LD	(IX+6*hcCTAC+1),A

hCTABY:		POP	BC
		RET
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoF hCTABC()

hcTABX:		POP	BC
		POP	DE
		POP	HL
		POP	IX
hCTABQ:		POP	AF
		RET
;================================================================= eoF hCTAB()

;::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: interrupt handler
		IF	uhCI		; combo interrupts

hcCIY		EQU	$		; copy to hi-ram start address
		.PHASE	hcCIC		; assemble to this address
hcICIS		EQU	$		; for code size calculation

;===================================================================== hICTA()
; Purpose.....:	Combo timer A interrupt handler
; Registers...: All saved
; Requires....: Nothing
; Labels......: hICTAx: h)ardware I)nterrupt C)ombo T)imer A)
;=============================================================================
hICTA::		PUSH	HL
		PUSH	AF

;----------------------------------------------- increment 32 bit ints counter
		LD	HL,(hvICTC)	; bump ints counter
		INC	HL
		LD	(hvICTC),HL
		LD	A,H		; check low word overflow
		OR	L
		JR	NZ,hICTAF	; no overflow, maintain flags
		LD	HL,(hvICTC+2)	; overflow, get hi word
		INC	HL		; bump it
		LD	(hvICTC+2),HL	; and store

;---------------------------------------------------------- set overflow flags
hICTAF:		XOR	A
		DEC	A		; set all elapsed flags
		LD	(hvICTF),A	; and store them

hICTAX:		POP	AF
		POP	HL
		EI
		RET			; may not be RETI, see chip docs

;------------------------------------------------------------------- variables
hvICTF::	DB	0		; timer elapsed flag
hvICTC::	DW	0		; ints counter low word
		DW	0		; hi word
;================================================================= eoI hICTA()

hcICIE		EQU	$		; for code size calculation
		.DEPHASE
		ENDIF			; uhCI
	
;############################################################### EOF LIBHC.INC
 calculation