;################################################################## LIBHSB.INC
; Purpose.....:	LIBH, SIO channel B functions. Included by LIBH.MAC
;#############################################################################

;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::: code for all modes
;====================================================================== hSBS()
; Purpose.....:	Setup SIO, channel B
; Input.......: A:    bit hbSFHR: use RTS handshake
;		      bit hbSFHD: use DTR handshake
;		B:    Baudrate to use, one of hcSRxx
;		C:    polling mode only: 0= no handshake wait, 1= use wait
;		CY=1: Init, set DTR & RTS
;               CY=0: Reset to default mode, 9600,N,8,1, no interrupts,
;		      so it can be used from CP/M as listing device again
; Registers...: None saved
; Requires....: hSBBS(), hSBHC()
; Labels......: hSBSxx: h)ardware S)IO channel B) S)etup
;=============================================================================
hSBS::		LD	E,A		; save init flags
 
;-------------------------------------------------------------- initialize SIO
		JR	NC,hSBSR	; reset if carry not set

		LD	A,B		; A= baudrate
		CALL	hSBBS		; set baud rate and handshake wait

		LD	HL,hvSBST	; HL= address of init table
		LD	B,(HL)		; B= length of table
		INC	HL		; HL= address of 1st table element
		LD	C,hcSPBC	; port to write to
		OTIR			; write data to SIO
		
;----------------------------------------------------------- reset error flags
		IF	uhSBPW		; polling only
		XOR	A
		LD	(hvSBWE),A	; reset hSBWE() error bits
		ENDIF
		IF	uhSBPR
		XOR	A
		LD	(hvSBRE),A	; reset hSBRE() error bits
		ENDIF

;------------------------------------------ initialize for interrupt operation
		IF	uhSBI		; interrupts

;-------------------------------------------------------- set interrupt vector
hcSBII		EQU	LOW hcIVT + LOW hcISV

		LD	A,2		; select WR2
		OUT	(hcSPBC),A	; write to control port B
		LD	A,hcSBII	; low byte of vector table
		OUT	(hcSPBC),A	; write to channel B

;---------------------------------------- copy interrupt code & data to hi-ram
		PUSH	DE		; save handshake flags
		LD	HL,hcSBIY	; start address in lo-ram
		LD	DE,hcSBIS	; start address hi-ram
		LD	BC,hcSBIE-hcSBIS; int code length
		LDIR			; copy the code
		POP	DE		; recall handshake flags

;----------------------------------------- copy interrupt enable bits to hiram
		IF	uhSBIW		; reqired only if TX interrupts
		LD	A,hcSBW1	; used to disable TX int
		LD	(hvISB1),A	; on empty buffer
		ENDIF			; uhSBIW

;----------------------------------------------------- setup interrupt vectors
hcSBSV		EQU	hcIVT+hcISV	; table start SIO interrupts

		IF	uhSBIR		; RX interrupts?
hcSBSR		EQU	hcSBSV+hcISBR	; table offset RX interrupt
hcSBSS		EQU	hcSBSV+hcISBS	; table offset RX spec cond interrupt

		LD	HL,hISBR	; get rx interrupt handler address
		LD	(hcSBSR),HL	; and write it to vector table
		LD	HL,hISBS	; get rx spec int handler address
		LD	(hcSBSS),HL	; and write it to vector table
		ENDIF			; uhSBIR

		IF	uhSBIW		; TX interrupts?
hcSBST		EQU	hcSBSV+hcISBT	; table offset TX interrupt
hcSBSE		EQU	hcSBSV+hcISBE	; table offset ext/status interrupt

		LD	HL,hISBW	; get tx interrupt handler address
		LD	(hcSBST),HL	; and write it to vector table
		LD	HL,hISBE	; get ext/status handler address
		LD	(hcSBSE),HL	; and write it to vector table
		ENDIF			; uhSBIW

		ENDIF			; uhSBI

;------------------------------------------ create handshake WR5 on/off values
		LD	A,E		; recall handshake flags
		CALL	hSBHC		; create and store WR5 on/off values
		SCF			; activate configured handshake lines
		CALL	hSBHS

		IF	uhSBIW		; tx ints?
		CALL	hSBHR		; read tx handshake status
		ENDIF			; uhSBIW

		RET

;---------------------------------------------------- SIO initialization table
		IF uhSBIR AND NOT uhSBIW
hcSBW1		EQU	01CH		; RX, and spec cond interrupts
		ENDIF
		IF uhSBIW AND NOT uhSBIR
hcSBW1		EQU	005H		; ext or TX interrupts
		ENDIF
		IF uhSBIW AND uhSBIR
hcSBW1		EQU	01DH		; RX, spec cond and ext interrupts
		ENDIF

hcSBW5		EQU	068H		; init value for WR5

		IFE	uhSBI		; polling mode
hvSBST:		DB	10		; size of table
		DB	018H		; channel reset
		DB	4, 044H		; WR4, clock divider, 1 stop, no par
		DB	1, 00H		; WR1, no interrupts
		DB	3, 0C1H		; WR3, 8 data, no autoenable, ena RCVR
		DB	5, hcSBW5	; WR5, 8 data, disable RTS & DTR
		DB	0C0H		; reset TX underrun error

		ELSE			; interrupt mode
hvSBST:		DB	11		; size of table
		DB	018H		; channel reset
		DB	038H		; return from interrupt
		DB	4, 044H		; WR4, clock divider, 1 stop, no par
		DB	1, hcSBW1	; WR1, ints according to uh defs
		DB	3, 0C1H		; WR3, 8 data, no autoenable, ena RCVR
		DB	5, hcSBW5	; WR5, 8 data, disable RTS & DTR
		DB	0C0H		; reset TX underrun error
		ENDIF			; uhSBI

;---------------------------------------- reset SIO to standard operating mode
hSBSR:		LD	HL,hvSBSR	; HL= address of init table
		LD	B,(HL)		; B= length of table
		INC	HL		; HL= address of 1st table element
		LD	C,hcSPBC	; port to write to
		OTIR			; write data to SIO

		LD	A,hcSR9K	; A= baudrate 9600
		CALL	hSBBS		; set baud rate

		RET

;------------------------------------------------------------ reset init table
hvSBSR:		DB	11		; size of table
		DB	018H		; channel reset
		DB	038H		; return from int
		DB	0C0H		; reset TX underrun error
		DB	4, 044H		; WR4, clock divider, 1 stop, no par
		DB	1, 00H		; WR1, no interrupts
		DB	3, 0C1H		; WR3, 8 data, no autoenable, ena RCVR
		DB	5, 068H		; WR5, 8 data, disable RTS & DTR
;================================================================== eoF hSBS()

;===================================================================== hSBBS()
; Purpose.....:	Set baudrate for SIO channel B
; Input.......: A: Baudrate to use, one of hcSRxx
;		C: 0= no handshake wait, 1= use handshake wait
; Registers...: A destroyed in polling mode, all saved otherwise
; Requires....: Nothing
; Labels......: hSBBSx: h)ardware S)IO channel B) B)audrate S)et
;=============================================================================
hSBBS::		PUSH	HL
		PUSH	DE
		PUSH	AF

		LD	HL,hvSBBR	; address of baudrate table
		LD	D,0
		RLCA			; each element is two bytes
		LD	E,A
		ADD	HL,DE		; HL= table address

;-------------------------------------------------------- setup baudrate timer
		LD	A,076H		; counter0 load lsb/msb, mode3, bin
		OUT	(hcSPTC),A	; write to 8253 control port
		LD	A,(HL)		; A= lobyte of counter value
		OUT	(hcSPT1),A	; write to counter0, low byte
		INC	HL
		LD	A,(HL)		; A= hibyte of counter value
		OUT	(hcSPT1),A	; write to counter0, hibyte
		POP	AF

;--------------------------------------------------------- setup RX loop count
		IFE	uhSBI		; only in polling mode

		LD	HL,0001H	; set default count
		LD	(hvSBBL),HL	; to 1 (inactive)

		BIT	0,C		; use counts?
		JR	Z,hSBBSX	; no, exit

		LD	HL,hvSBBC	; address of counts table
		LD	D,0
		RLCA			; each element is two bytes
		LD	E,A
		ADD	HL,DE		; HL= table address

		LD	A,(HL)
		LD	(hvSBBL),A	; store lobyte of loop count
		INC	HL
		LD	A,(HL)
		LD	(hvSBBL+1),A	; store hibyte of loop count
		ENDIF			; uhSBI/

hSBBSX:		POP	DE
		POP	HL
		RET

;----------------------------------------------------- baudrate counter values
hvSBBR::	DW	082EH		; 110 baud
		DW	0300H		; 300 baud
		DW	0180H		; 600 baud
		DW	00C0H		; 1200 baud
		DW	0060H		; 2400 baud
		DW	0030H		; 4800 baud
		DW	0018H		; 9600 baud
		DW	000CH		; 19200 baud

;----------------------------------------------------------- read loop counts
		IFE	uhSBI		; in polling mode noly
hvSBBC::	DW	1400H		; 110 baud
		DW	0A00H		; 300 baud
		DW	0500H		; 600 baud
		DW	03A0H		; 1200 baud
		DW	0150H		; 2400 baud
		DW	00A8H		; 4800 baud
		DW	0054H		; 9600 baud
		DW	002AH		; 19200 baud
		ENDIF			; uhSBI/

;------------------------------------------------------------- local variables
		IFE	uhSBI		; only used in polling mode
hvSBBL:		DW	0001H		; read loop count to use
		ENDIF			; uhSBI/
;================================================================= eoF hSBBS()

;===================================================================== hSBHC()
; Purpose.....: Configure rx handshake use channel B
; Input.......: A: hbSFHR (RTS/CTS) and/or hbSFHD (DTR/DSR)
; Registers...: All saved
; Requires....: Nothing
; Labels......: hSBHCx: h)ardware S)IO channel B) H)andshake C)onfigure
;=============================================================================
hSBHC::		PUSH	HL
		PUSH	DE
		PUSH	AF

		LD	(hvSBHS),A	; save for hSBWR()

		LD	E,00H		; AND mask for TX (CTS/DSR)
		LD	H,hcSBW5	; W5 init value
		LD	L,H		; same to L

		BIT	hbSFHR,A	; RTS/CTS?
		JR	Z,hSBHCD	; no
		SET	1,H		; yes, set RTS OR mask
		RES	1,L		; and RTS AND mask
		SET	5,E		; CTS AND mask

hSBHCD:		BIT	hbSFHD,A	; DTR/DSR?
		JR	Z,hSBHCW	; no
		SET	7,H		; yes, set DTR OR mask
		RES	7,L		; and DTR AND mask
		SET	3,E		; DSR AND mask

hSBHCW:		LD	(hvSB5D),HL	; write the RX masks
		LD	A,E		; A= TX mask
		LD	(hvSBHM),A

		IF	uhSBIR		; RX in interrupt mode
		LD	A,L		; A= HS off value
		LD	(hvISB5),A	; write to hi-ram
		ENDIF			; uhSBIR

		IF	uhSBIW		; TX in interrupt mode
		LD	A,E		; A= CTS/DSR AND mask
		LD	(hvISBM),A	; write to hi-ram
		ENDIF			; uhSBIW

hSBHCX:		POP	AF
		POP	DE
		POP	HL
		RET

;------------------------------------------------------------- local variables
hvSB5D:		DB	00		; disable (deactivate) HS WR5 value
hvSB5E:		DB	00		; enable (activate) HS WR5 value
hvSBHS:		DB	00		; handshake flags for hSBWR()
hvSBHM:		DB	00		; AND Mask for TX handshake
;================================================================= eoF hSBHC()
		
;===================================================================== hSBHS()
; Purpose.....: Set/Reset configured handshake line(s) channel B
; Input.......: CY=1: Set line(s) (active)
;               CY=0: Reset lines(s) (inactive)
; Registers...: All saved
; Required....: Nothing
; Labels......: hSBHSx: h)ardware S)IO channel B) H)andshake S)et
;=============================================================================
hSBHS::		PUSH	AF

		JR	C,hSBHSE	; if CY go and set active

hSBHSD:		LD	A,(hvSB5D)	; A= HS disable WR5 value
		JR	hSBHSW		; go to write

hSBHSE:		LD	A,(hvSB5E)	; A= HS enable WR5 value

hSBHSW:		PUSH	AF		; save new value
		LD	A,5		; address WR5
		OUT	(hcSPBC),A
		POP	AF		; get new value back
		OUT	(hcSPBC),A	; and set it

		POP	AF
		RET
;================================================================= eoF hSBHS()

;===================================================================== hSBHR()
; Purpose.....: Read CTS/DSR handshake status. If tx ints are used, writes
;		on/off status to hi-ram. Used on app startup
; Output......: CY=1: handshake enabled, CY=0: it is not
; Registers...: A destroyed
; Requires....: Nothing
; Labels......: hSBHRx: h)ardware S)IO channel B) H)andshake R)ead
;=============================================================================
hSBHR::		PUSH	BC
		PUSH	DE

		LD	B,0		; used for result

		LD	A,010H		; reset ext/status int command
		OUT	(hcSPBC),A	; send to SIO
		IN	A,(hcSPBC)	; get SIO status
		LD	C,A		; save for masking
		LD	A,(hvSBHM)	; A= used tx handshake lines
		LD	E,A		; save for compare		
		AND	C		; mask with status
		CP	E		; must be same as used lines 
		JR	NZ,hSBHRI	; not enabled, update int flags
		SET	0,B		; it's on, set flag

hSBHRI:
		IF	uhSBIW		; tx ints?
		LD	A,(hvISBF)	; get tx int flags
		RES	hbISBH,A	; preset to off
		BIT	0,B		; is it on?
		JR	Z,hSBHRW	; no
		SET	hbISBH,A	; yes, set flag
hSBHRW:		LD	(hvISBF),A	; save int flags
		ENDIF			; uhSBIW

		RR	B		; set result (CY)

		POP	DE
		POP	BC
		RET
;================================================================= eoF hSBHR()

;::::::::::::::::::::::::::::::::::::::: code for polling mode only, RX and TX
		IF uhSBP		; polling read or write
;==================================================================== hSBPKS()
; Purpose.....: Configure abort key channel B
; Input.......: A: abort key for RX/TX (0= no keyboard check)
; Registers...: AF destroyed 
; Requires....: Nothing
; Labels......: hSBPKS: h)ardware S)IO channel B) P)olling K)ey abort S)et
;=============================================================================
hSBPKS::	LD	(hvSBKA),A	; store abort key
		OR	A		; check for zero (disabled)
		LD	A,0C9H		; preset to disabled, set RET code
		JR	Z,hSBPKX	; disabled, set RET code
		XOR	A		; enabled, set NOP code

hSBPKX:		LD	(hSBPKC+1),A	; modify code
		RET

;------------------------------------------------------------- local variables
hvSBKA:		DB	00		; RX/TX abort key
;================================================================ eoF hSBPKS()

;==================================================================== hSBKKC()
; Purpose.....: Check for abort request, if abort is enabled
; Output......: A: Code of pressed key, 0 if no key or not enabled
; 		CY: 1 if abort request, 0 if not enabled or no request		
; Registers...: AF destroyed
; Requires....: hKR()
; Labels......: hSBPKC: h)ardware S)IO channel B) P)olling K)ey abort C)heck
;=============================================================================
hSBPKC::	XOR	A		; clear carry in case not enabled
		RET			; this will be overwritten with NOP
					; if abort gets enabled
		PUSH	BC
		CALL	hKR		; read keyboard using MOS
		LD	B,A		; save key (if any)
		JR	Z,hSBPKY	; no key, exit (CY=0 if no MOS err)

		LD	A,(hvSBKA)	; get abort key
		CP	B		; abort key pressed?
		SCF			; preset to abort pressed
		JR	Z,hSBPKY	; if pressed, exit with CY=1
		XOR	A		; nothing or something else, CY=0

hSBPKY:		LD	A,B		; A= pressed key
		POP	BC
		RET
;================================================================= eoF hSBKC()
		ENDIF			; uhSBP

;:::::::::::::::::::::::::::::::::::: code for polling and interrupt, RX only
		IF	uhSBPR OR uhSBIR
;===================================================================== hSBRC()
; Purpose.....: Clear SIO RX buffer, channel B
; Registers...: All saved
; Requires....: hSBR() or hSBRI()
; Labels......: hSBRCx: h)ardware S)IO channel B) R)x C)lear
;=============================================================================
hSBRC::		PUSH	AF

		IF	uhSBIR		; interrupt mode
hSBRCL:		CALL	hSBIR		; get byte from SIO int buffer
		ELSE
hSBRCL:		CALL	hSBPR		; get byte from SIO
		ENDIF			; uhSBIR

		JR	NZ,hSBRCL	; until none ready

		POP	AF
		RET
;================================================================= eoF hSBRC()

;===================================================================== hSBRE()
; Purpose.....: Read error bits of last read operation and clear, CH-B
; Output......: A: Error bits
;                  bit4 = parity error
;                  bit5 = receiver overrun
;                  bit6 = framing error        
;		Z=0: some error bits set
;		Z=1: no error (A=0)
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hSBREx: h)ardware S)IO channel B) R)ead E)rror
;=============================================================================
hSBRE::		PUSH	HL

		IF	uhSBPR
		LD	HL,hvSBRE
		ENDIF
		IF	uhSBIR
		LD	HL,hvISBS
		ENDIF

		LD	A,(HL)		; get last error bits
		OR	A		; no error?
		JR	Z,hSBREX	; yes, nothing to do
		LD	(HL),0		; no, clear errors


hSBREX:		POP	HL
		RET

;------------------------------------------------------------------ local data
hvSBRE:		DB	0		; read error bits
;================================================================= eoF hSBRE()
		ENDIF			; uhSBPR OR uhSBIR

;::::::::::::::::::::::::::::::::::::: code for polling and interrupt, TX only
		IF	uhSBPW OR uhSBIW
;===================================================================== hSBWE()
; Purpose.....:	Read transmit underrun bit and clear it if set, channel B
; Output......: A: Error bits, bit6 = transmit underrun
;		Z=1: no error
;		Z=0: error
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hSBWEx: h)ardware S)IO channel B) W)rite E)rror
;=============================================================================
hSBWE::		PUSH	HL

		IF	uhSBPW
		LD	HL,hvSBWE
		ENDIF
		IF	uhSBIW
		LD	HL,hvISBE
		ENDIF

		LD	A,(HL)		; get last error bits
		OR	A		; no error?
		JR	Z,hSBWEX	; yes, nothing to do
		LD	(HL),0		; clear errors

hSBWEX:		POP	HL
		RET

;------------------------------------------------------------------ local data
hvSBWE:		DB	0		; tx error flags
;================================================================= eoF hSBWE()
		ENDIF			; uhSBPW OR uhSBIW

;::::::::::::::::::::::::::::::::::::::::::::::::::: code for polling, RX only
		IF	uhSBPR
;====================================================================== hSBR()
; Purpose.....: Check receive status, read byte from SIO channel B
;		When connected to a Windows target, it continues to send for
;		an undefined time after handshake was deactivated. This leads
;		to buffer overruns and loss of receive bytes. So we wait until
;		the sender is silent and buffer the received bytes.
; Output......: A:    received byte
;               CY=1: local buffer overrun
;               CY=0: no overrun
;               Z=1:  no byte received      
;               Z=0:  byte received
; Registers...: AF destroyed
; Requires....:	Nothing
; Labels......:	hSBPRx: h)ardware S)IO channel B) P)olling R)ead
;=============================================================================
hSBPR::		PUSH	IX
		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	B,0		; clear return flags 

;------------------------------------------------------ check SIO error status
		LD	A,1		; select read register 1
		OUT	(hcSPBC),A
		IN	A,(hcSPBC)	; read register
		AND	070H		; mask error bits
		JR	Z,hSBPRC	; there are no errors
		LD	(hvSBRE),A	; save error bits
		LD	A,030H		; error reset command
		OUT	(hcSPBC),A	; write to SIO

;--------------------------- check if buffer empty, if not, return buffer byte
hSBPRC:		LD	IX,hvSBRC	; IX pointes to buffer counter
		LD	A,(IX+0)	; get it
		OR	A		; is buffer empty?
		JR	NZ,hSBPRR	; no, return next buffer byte

;------------------------------------------------------------- handshake pulse
		SCF
		CALL	hSBHS
		LD	DE,(hvSBBL)	; pulse width, depends on baudrate
hSBPRO:		DEC	DE
		LD	A,E
		OR	D
		JR	NZ,hSBPRO
		XOR	A
		CALL	hSBHS

;------------------------------------------ read SIO data until FIFO is empty
		LD	HL,hvSBRB	; HL= buffer address
		XOR	A
		LD	(IX+0),A	; reset buffer counter
	
hSBPRN:		LD	DE,(hvSBBL)	; loop count, depends on baudrate

hSBPRL:		IN	A,(hcSPBC)	; read status
		RRA			; RX ready to CY
		JR	C,hSBPRW	; RX FIFO not empty

		DEC	DE		; wait as long as a byte needs
		LD	A,E		; to enter the SIO, depends on
		OR	D		; baudrate
		JR	NZ,hSBPRL	; time not over, reread

		LD	HL,hvSBRB	; reset read pointer
		LD	(hvSBRR),HL	; to buffer start
		JR	hSBPRR		; time over, nothing more, read buffer

;-------------------------------------------- a byte is ready, write to buffer
hSBPRW:		IN	A,(hcSPBD)	; get byte from SIO
		LD	(HL),A		; and save in buffer
		INC	HL		; bump pointer
		INC	(IX+0)		; bump counter
		LD	A,hcSBRZ	; A= buffer size
		CP	(IX+0)		; max reached?
		JR	NZ,hSBPRN	; no, check for next

;------------------------------------------------------- local buffer overflow
		SET	0,B		; signal to caller

;-------------------------------------- read local buffer and return next byte
hSBPRR:		LD	A,(IX+0)	; get buffer counter
		OR	A		; is buffer empty?
		JR	Z,hSBPRX	; yes, exit

hSBPRB:		LD	HL,(hvSBRR)	; where to read next byte
		LD	A,(HL)		; get it
		INC	HL		; bump pointer
		LD	(hvSBRR),HL	; and save it
		SET	1,B		; signal char ready
		DEC	(IX+0)		; dec buffer counter

hSBPRX:		RR	B		; C=1 buffer overflow
		BIT	0,B		; Z=0 char received

		POP	BC
		POP	DE
		POP	HL
		POP	IX
		RET

;------------------------------------------------------------- local variables
hcSBRZ		EQU	32		; buffer size
hvSBRC:		DB	0		; buffer fill counter
hvSBRR:		DW	hvSBRB		; buffer read pointer
hvSBRB::	DS	hcSBRZ		; read-ahead buffer
;================================================================= eoF hSBPR()
		ENDIF			; uhSBPR

;:::::::::::::::::::::::::::::::::::::::::::::::::: code for polling, TX only
		IF	uhSBPW
;===================================================================== hSBPW()
; Purpose.....: Check handshake if enabled, tx pending, send byte, channel B
; Input.......: A:    Byte to send
; Output......: CY=0: Byte sent
;               CY=1: Abort was pressed while waiting for handshake/TX empty
;		Z=1:  Handshake not ready if no-wait is configured
; Registers...: All saved
; Requires....: hSBPKC()
; Labels......: hSBPWx: h)hardware S)IO channel B) P)olling W)rite
;=============================================================================
hSBPW::		PUSH	BC
		
		LD	C,A		; save char to send

;----------------------------------------------- check transmit underrun error
		IN	A,(hcSPBC)	; get SIO status
		BIT	6,A		; transmit underrun?
		JR	Z,hSBPWC	; no, no need to reset it

		LD	(hvSBWE),A	; yes, save it
		LD	A,0C0H		; reset TX underrun command
		OUT	(hcSPBC),A	; write it to SIO

;------------------------------------------------- wait until TX FIFO is empty
hSBPWC:		LD	A,1		; select read register 1
		OUT	(hcSPBC),A	; write to control register
		IN	A,(hcSPBC)	; read read register 1
		RR	A		; bit 0 (all sent) to carry
		JR	C,hSBPWL	; all sent, continue
		CALL	hSBPKC		; check abort request
		JR	C,hSBPWX	; CY set, abort
		JR	hSBPWC		; continue checking

;---------------------------------------------------- check for keyboard abort
hSBPWL:		CALL	hSBPKC		; check for abort request
		JR	C,hSBPWX	; CY set, abort

;------------------------------------------------------ check handshake active
		CALL	hSBHR		; get handshake status
		JR	C,hSBPWW	; active, write to SIO
		LD	A,(hvSBWH)	; wait for handshake?
		OR	A
		JR	Z,hSBPWX	; no, exit with Z=1
		JR	hSBPWL		; yes, continue waiting
		
;-------------------------------------------------- finally, write data to SIO
hSBPWW:		LD	A,C		; recall send byte
		OUT	(hcSPBD),A	; and send it
		XOR	A		; clear CY, byte sent
		INC	A		; Z=0, handshake on 

hSBPWX:		POP	BC
		RET
;================================================================= eoF hSBPW()

;==================================================================== hSBPWH()
; Purpose.....:	Set wait-for-handshake mode for transmit in polling mode
; Input.......: CY=1: Wait, CY=0 do not
; Registers...: All saved
; Requires....: Nothing
; Labels......: hSBPWH: h)ardware S)IO channel B) P)olling W)rite H)andshake
;=============================================================================
hSBPWH::	PUSH	AF

		LD	A,0
		RL	A
		LD	(hvSBWH),A

		POP	AF
		RET

;------------------------------------------------------------- local variables
hvSBWH:		DB	0		; wait for handshake flag
;================================================================ eoF hSBPWH()
		ENDIF			; uhSBPW

;:::::::::::::::::::::::::::::::::::::::::::: code used in interrupt mode only
		IF	uhSBI
;::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: interrupt RX only
		IF	uhSBIR
;===================================================================== hSBHI()
; Purpose.....: Maintain hanshake-on in interrupt operation mode
; Registers...: All saved
; Requires....: hSBHS()
; Labels......: hSBIHx: h)ardware S)IO channel B) I)nterrupt H)andshake
;=============================================================================
hSBIH::		PUSH	AF

;--------------------------------------- check if buffer level above off-level 
		LD	A,(hvISBH)	; get buffer-fill counter
		CP	hcSBIN		; is it below on-level?
		JR	NC,hSBIHX	; no, above or equal, exit

;--------------------------------- check if handshake was set off in interrupt
		LD	A,(hvISBR)	; A= int has reset flag
		OR	A
		JR	Z,hSBIHX	; was not reset, exit

;-------------- it was reset and is below on-level, set handshake active again
		XOR	A
		LD	(hvISBR),A	; clear int-reset flag
		SCF
		CALL	hSBHS		; set handshake line(s)

hSBIHX:		POP	AF
		RET
;================================================================= eoF hSBIH()

;===================================================================== hSBIR()
; Purpose.....: Check RX interrupt buffer, read char if one is ready
; Output......: A:    received byte
;               Z=1:  no byte received      
;               Z=0:  byte received
;		CY=1: int rx buffer overflow
;		CY=0: no overflow
; Registers...: AF destroyed
; Requires....:	hSBIH()
; Labels......:	hSBIRx: h)ardware S)IO channel B) I)nterrupt R)ead
;=============================================================================
hSBIR::		DI
		PUSH	HL
		PUSH	DE

;------------------------------------------------ check if int RX buffer empty
		LD	A,(hvISBH)	; get buffer fill level
		LD	D,A		; save for handshaking
		OR	A
		JR	Z,hSBIRQ	; it's empty, exit

;---------------------------------------------------- adjust buffer fill level
		LD	A,D		; dec fill level
		DEC	A
		LD	(hvISBH),A	; write to int var

;-------------------------------------------- buffer not empty, read next byte
		LD	HL,(hvSBRB)	; get buffer read pointer
		LD	E,(HL)		; get byte from RX buffer
		INC	HL		; bump read pointer

;--------------------------------------------------------- buffer pointer wrap
		LD	A,(hvSBRC)	; get buffer usage
		INC	A		; bump it
		CP	hcSBIZ		; size reached?
		JR	NZ,hSBIRU	; no, go to udate

		LD	HL,hcSBIB	; yes, reset pointer
		XOR	A		; and counter

hSBIRU:		LD	(hvSBRB),HL	; update buffer pointer
		LD	(hvSBRC),A	; and counter

;-------------------------------------- maintain handshake and buffer overflow
hSBIRZ:		CALL	hSBIH		; check and set handshake-on

		LD	A,(hvISBO)	; get buffer overflow flag
		RR	A		; CY=1, overflow
		LD	A,0
		LD	(hvISBO),A	; reset overflow flag
		INC	A		; Z=0, byte received

hSBIRQ:		LD	A,E		; recall buffer byte
		POP	DE
		POP	HL
		EI
		RET

;------------------------------------------------------------- local variables
hvSBRC:		DB	0		; buffer read counter
hvSBRB:		DW	hcSBIB		; current read buffer address
;================================================================= eoF hSBIR()
		ENDIF			; uhSBIR
		IF	uhSBIW		; TX interrupts
;===================================================================== hSBIW()
; Purpose.....:	Write a byte to tx interrupt buffer, if empty to SIO
; Input.......: A: Byte to write
; Output......: CY=1: tx buffer full
;		CY=0: byte written
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hSBIWx: h)ardware S)IO channel B) I)nterrupt W)rite
;=============================================================================
hSBIW::		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	E,A		; save send-byte
		DI

		LD	A,(hvISBF)	; get int flags
		LD	B,A		; save for processing
		LD	A,(hvISBT)	; A= buffer fill counter
		LD	C,A		; save to C

;---------------------------------------------------------- check ints running
hSBIWC:		BIT	hbISBI,B	; are ints running?
		JR	NZ,hSBIWB	; yes, write to buffer

;---------------- check handshake, if off, write to buffer and flag to ext int
hSBIWH:		BIT	hbISBH,B	; is handshake on?
		JR	NZ,hSBIWI	; yes, enable ints and send
		BIT	hbISBU,B	; no, 1st unsent already flagged?
		JR	NZ,hSBIWB	; yes, just write to buffer
		SET	hbISBU,B	; no, flag to ext int byte is ready
		LD	A,(hvISBW)	; A= buffer write index
		LD	(hvISBU),A	; save for ext int if HS gets active 
		JR	hSBIWB		; write byte to buffer

;------------------------- ints off, handshake on, start ints and write to SIO
hSBIWI:		SET	hbISBI,B	; set ints-running flag
		LD	A,1		; select WR1
		OUT	(hcSPBC),A	; write to control reg
		LD	A,hcSBW1	; get int-enable bits
		SET	1,A		; set tx int enable
		OUT	(hcSPBC),A	; write it
		LD	A,E		; recall send byte
		OUT	(hcSPBD),A	; write to SIO
		XOR	A		; signal byte sent (CY=0)
		JR	hSBIWX		; and exit

;----------------------------------------------------------- check buffer full
hSBIWB:		LD	A,hcSBIX	; A= buffer size
		CP	C		; check for full
		SCF			; preset to full
		JR	Z,hSBIWX	; it's full, exit with CY set

;---------------------------------------- write byte to buffer, maintain index
		PUSH	BC		; save flags and counter
		LD	A,(hvISBW)	; A= buffer write index
		LD	C,E		; C= send byte
		LD	D,0		; D= 0 for ADD
		LD	E,A		; E= write index for ADD
		LD	HL,hcSBIU	; HL= buffer base address
		ADD	HL,DE		; HL= actual buffer address
		LD	(HL),C		; write send byte to buffer
		INC	A		; bump write index
		CP	hcSBIX		; size reached?
		JR	NZ,hSBIWA	; no, save
		XOR	A		; yes, reset to zero
hSBIWA:		LD	(hvISBW),A	; save write index

		POP	BC		; recall flags and counter
		INC	C		; bump buffer fill counter
		LD	A,C
		LD	(hvISBT),A	; and save it
		XOR	A		; CY=0, signal byte sent

hSBIWX:		LD	A,B
		LD	(hvISBF),A	; save new flags
	
		POP	BC
		POP	DE
		POP	HL
		EI
		RET
;================================================================= eoF hSBIW()
		ENDIF			; uhSBIW

;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::: interrupt handlers
hcSBIY		EQU	$		; copy to hi-ram start address
		.PHASE	hcSBIC
hcSBIS		EQU	$		; for code size calculation

;=============================================================================
; Due to the address decoding on the serial card, which prevents the decoding
; of the RETI by the SIO, all interrupt routines must be terminated with a 
; 'return from int' command to the SIO.
;=============================================================================
		IF	uhSBIR		; RX interrupt
;===================================================================== hSBIR()
; Purpose.....: SIO-B RX interrupt handler
; Registers...: All saved
; Requires....: Nothing
; Labels......: hISBRI: h)hardware I)nt handler S)IO channel B) R)ead
;=============================================================================
hISBR::		PUSH	HL
		PUSH	AF

;----------------------------------------- read received byte, store in buffer
		LD	HL,(hvISBB)	; get buffer address
		IN	A,(hcSPBD)	; get received byte
		LD	(HL),A		; store in buffer
		INC	HL		; bump pointer

;------------------------------------ update counter and pointer, wrap if full
		LD	A,(hvISBC)	; get buffer counter
		INC	A		; bump it
		CP	hcSBIZ		; size reached?
		JR	NZ,hISBRW	; no, write values back
	
		LD	HL,hcSBIB	; yes, go to buffer start
		XOR	A		; reset buffer counter

hISBRW:		LD	(hvISBB),HL	; write new buffer pointer
		LD	(hvISBC),A	; write new buffer counter

;------------------------------------------------------- check buffer overflow
		LD	A,(hvISBH)	; get handshake-off counter
		INC	A		; bump it
		LD	L,A		; save for handshake
		LD	(hvISBH),A	; and write back
		CP	hcSBIZ		; buffer full?
		JR	C,hISBRH	; no, continue
		LD	A,1		; yes, buffer overflowed
		LD	(hvISBO),A	; set buffer overflow flag

;---------------------------------------- reset handshake if off-level reached
hISBRH:		LD	A,L		; get handshake-off counter
		CP	hcSBIO		; off-level reached?
		JR	C,hISBRX	; no, exit

		LD	A,(hvISBR)	; is it already reset?
		OR	A
		JR	NZ,hISBRX	; yes, exit

		LD	A,5		; select WR5
		OUT	(hcSPBC),A	; write to control register
		LD	A,(hvISB5)	; get WR5 value for HS off
		OUT	(hcSPBC),A	; and write to SIO

		LD	A,1
		LD	(hvISBR),A	; set handshake-is-reset flag

hISBRX:		LD	A,038H		; reset SIO interrupt
		OUT	(hcSPAC),A	; must use CH-A control port

		POP	AF
		POP	HL
		EI
		RETI

;------------------------------------------------------------- local variables
hvISBB:		DW	hcSBIB		; current buffer address
hvISBC:		DB	0		; buffer usage counter
hvISBO:		DB	0		; buffer overflow flag
hvISBH:		DB	0		; handshake-off counter
hvISB5:		DB	0		; value to write to WR5 for HS-off
hvISBR:		DB	0		; handshake was reset flag
;================================================================= eoI hISBR()

;===================================================================== hISBS()
; Purpose.....: Channel-B RX special condition interrupt service
; Registers...: All saved
; Requires....: Nothing
; Labels......: hISBSx: h)hardware I)nt S)IO ch B) S)pecial receive condition
;=============================================================================
hISBS::		PUSH	AF

		IN	A,(hcSPBD)	; data must be read, even if error
 
		LD	A,1		; select read register 1
		OUT	(hcSPBC),A	; write to control register
		IN	A,(hcSPBC)	; get (error) status
		LD	(hvISBS),A	; and store it
		LD	A,030H		; reset-error command
		OUT	(hcSPBC),A	; send to SIO

hISBSX:		LD	A,038H		; reset SIO interrupt
		OUT	(hcSPAC),A	; must use channel A

		POP	AF
		EI
		RETI
;================================================================= eoI hISBS()
		ENDIF			; uhSBIR
		IF	uhSBIW		; TX interrupts
;===================================================================== hSBIW()
; Purpose.....:	Transmit one byte in interrupt mode
; Requires....: Nothing
; Labels......: hISBWx: h)ardware I)nt S)IO channel B) W)rite
;=============================================================================
hISBW::		PUSH	AF
		PUSH	BC
		PUSH	DE
		PUSH	HL

		LD	A,(hvISBF)	; get int flags
		LD	B,A		; save

;---------------------------------------------------------- check buffer empty
hISBWC:		LD	A,(hvISBT)	; get buffer fill counter
		LD	C,A		; save for later
		OR	A		; is it empty?
		JR	Z,hISBWD	; yes, disable TX interrupts, exit

;---------------------------------------------------------- check handshake on
		BIT	hbISBH,B	; is handshake on?
		JR	Z,hISBWN	; no, store next byte for ext int

;--------------------------------------- get byte from buffer and write to SIO
hISBWB:		LD	A,(hvISBA)	; A= buffer read index
		LD	E,A		; E= buffer read index
		LD	D,0		; D=0 for ADDs
		LD	HL,hcSBIU	; HL= buffer base address
		ADD	HL,DE		; HL= address of next byte
		LD	A,(HL)		; A= byte to send
		BIT	hbISBS,B	; was byte already sent from ext int?
		RES	hbISBS,B	; reset flag
		JR	NZ,hISBWI	; yes, go to serve index & cnt
		OUT	(hcSPBD),A	; no, write to SIO

hISBWI:		LD	A,(hvISBA)	; A= read index
		INC	A		; bump it
		CP	hcSBIX		; size reached?
		JR	NZ,hISBWS	; no, save
		XOR	A		; yes, reset to zero

hISBWS:		LD	(hvISBA),A	; write it back
		DEC	C		; dec buffer counter
		LD	A,C		; A= buffer counter
		LD	(hvISBT),A	; save it
		RES	hbISBN,B	; reset send-on-HSon flag for ext int
		JR	hISBWX		; exit interrupt

;------------------------- store next-to-send byte for HS-on event in ext int
hISBWN:		LD	A,(hvISBA)	; buffer index for next byte
		LD	E,A		; save for ADD
		LD	D,0		; zero for ADD
		LD	HL,hcSBIU	; HL= buffer base address
		ADD	HL,DE		; HL= byte address 1+ byte
		LD	A,(HL)		; get it
		LD	(hvISBN),A	; store for ext int in case HS on
		SET	hbISBN,B	; tell ext-int to send on HS on
		BIT	hbISBH,B	; is handshake on?
		JR	NZ,hISBWX	; yes, exit interrupt

;-------------------------------------- buffer is empty, disable TX interrupts
hISBWD:		LD	A,1		; select WR1
		OUT	(hcSPBC),A	; write to control reg
		LD	A,(hvISB1)	; get int-enable bits
		RES	1,A		; reset tx int enable bit
		OUT	(hcSPBC),A	; write it
		RES	hbISBI,B	; reset int-enabled flag

hISBWX:		LD	A,B
		LD	(hvISBF),A	; save the new flags

		LD	A,038H		; reset SIO interrupt
		OUT	(hcSPAC),A	; must use channel A

		POP	HL
		POP	DE
		POP	BC
		POP	AF
		EI
		RETI
;================================================================= eoI hISBW()

;===================================================================== hISBE()
; Purpose.....:	Ext/Status interrupt handler
; Requires....: Nothing
; Labels......: hISBEx: h)ardware I)nt S)IO channel B) E)xt/status
;=============================================================================
hISBE::		PUSH	AF
		PUSH	BC

;-------------------------------------------- reset ext int and get SIO status
		LD	A,010H		; reset ext/status interrupt command
		OUT	(hcSPBC),A	; write it
		IN	A,(hcSPBC)	; get new status
		LD	C,A		; save it
		LD	(hvISBE),A	; and store it for application

;-------------------------------------------- check if tx handshake is enabled
		LD	A,(hvISBF)	; get flags
		LD	B,A		; and save them

		PUSH	BC
		LD	A,(hvISBM)	; get used handshake line(s)
		LD	B,A		; save for compare
		AND	C		; mask with status
		CP	B		; must be same as used lines 
		POP	BC
		JR	NZ,hISBER	; not enabled, reset on-flag, exit

;------------------ handshake got enabled, enable tx ints if something to send 
		BIT	hbISBH,B	; handshake already on?
		JR	NZ,hISBEX	; yes, not config'd line caused int

		SET	hbISBH,B	; no, set tx handshake-on flag
		BIT	hbISBN,B	; something to send?
		RES	hbISBN,B	; reset flag
		JR	NZ,hISBEN	; yes, sent next byte

;----------- there could be data in buffer, HS was off while buffer got filled 
		BIT	hbISBU,B	; unsent data in buffer?
		JR	Z,hISBEX	; no, exit
		RES	hbISBU,B	; yes, reset flag	
		LD	HL,hcSBIU	; HL= buffer base address
		LD	D,0
		LD	A,(hvISBU)	; A= index of 1st not sent byte
		LD	E,A
		ADD	HL,DE
		LD	A,(HL)		; get buffer byte
		LD	(hvISBN),A	; save for send below
	
;------------------------------- enable tx ints and send next byte from buffer
hISBEN:		LD	A,1		; select WR1
		OUT	(hcSPBC),A	; write to control register
		LD	A,(hvISB1)	; get WR1 image
		SET	1,A		; set tx int enable
		OUT	(hcSPBC),A	; write to SIO
		SET	hbISBS,B	; signal to tx int byte was sent
		SET	hbISBI,B	; set ints running flag
		LD	A,(hvISBN)	; get next byte to send
		OUT	(hcSPBD),A	; write to data register
		JR	hISBEX

hISBER:		RES	hbISBH,B	; reset handshake-on flag

hISBEX:		LD	A,B
		LD	(hvISBF),A	; store the new flags

		LD	A,038H		; reset SIO interrupt
		OUT	(hcSPAC),A	; must use channel A

		POP	BC
		POP	AF
		EI
		RETI
;================================================================= eoI hISBE()

;====================================================== tx interrupt variables
;--------------------------------------------- flag bits definition for hvISBF
hbISBI		EQU	0		; tx I)nts running
hbISBN		EQU	1		; send N)ext byte on handshake-on
hbISBH		EQU	2		; tx H)andshake CTS and/or DSR is on
hbISBS		EQU	3		; a byte was S)ent from ext int
hbISBU		EQU	4		; U)nsent data in buffer

hvISBA:		DB	0		; tx buffer rA)ad index
hvISBW:		DB	0		; tx buffer W)rite index
hvISBT:		DB	0		; tx buffer fill counT)er
hvISB1:		DB	0		; WR1) image
hvISBF:		DB	0		; tx int F)lags
hvISBM:		DB	0		; tx CTS/DSR AND M)ask
hvISBN:		DB	0		; byte to send on handshake-oN)
hvISBE:		DB	0		; SIO status after ext interrupt
hvISBU:		DB	0		; buffer index of 1st U)nsent byte
hvISBS:		DB	0		; receive error flags
;================================================== end tx interrupt variables
		ENDIF			; uhSBIW

hcSBIE		EQU	$		; for code size calculation
		.DEPHASE

		ENDIF			; uhSBI

;############################################################## EOF LIBHSB.INC
alculation
		.DEPHASE

		ENDIF			; uhSBI

;############################################################## EOF LIBH